//
//  RFIDController.m
//  RFID
//
//  Created by Phidgets on 2015-07-17.
//  Copyright (c) 2015 Phidgets. All rights reserved.
//

#import "RFIDController.h"

@implementation RFIDController

- (void)windowWillClose:(NSNotification*)aNotification {
    if(ch){
        //Stop all events before closing
        Phidget_setOnAttachHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_setOnDetachHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_setOnErrorHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_close((PhidgetHandle)ch);
        PhidgetRFID_delete(&ch);
        ch = NULL;
    }
}

-(void)awakeFromNib{
    PhidgetReturnCode result;
    
    //Manage GUI
    NSRect frame = [mainWindow frame];
    frame.size.height = [mainWindow minSize].height;
    [mainWindow setFrame:frame display:NO];
    [mainWindow center];
    [mainWindow setDelegate:self];
    [phidgetInfoBoxView fillPhidgetInfo:nil];
    [phidgetInfoBox setHidden:NO];
    
    result = PhidgetRFID_create(&ch);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    
    result = [self initChannel:(PhidgetHandle)ch];
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    
    result = PhidgetRFID_setOnTagHandler(ch, gotTagData, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    
    result = PhidgetRFID_setOnTagLostHandler(ch, gotTagLostData, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    
    [PhidgetInfoBox openCmdLine:(PhidgetHandle)ch];
}

-(PhidgetReturnCode)initChannel:(PhidgetHandle) channel{
    PhidgetReturnCode result;
    
    result = Phidget_setOnAttachHandler(channel, gotAttach, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        return result;
    }
    
    result = Phidget_setOnDetachHandler(channel, gotDetach, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        return result;
    }
    
    result = Phidget_setOnErrorHandler(channel, gotError, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        return result;
    }
    
    /*
     * Please review the Phidget22 channel matching documentation for details on the device
     * and class architecture of Phidget22, and how channels are matched to device features.
     */
    
    /*
     * Specifies the serial number of the device to attach to.
     * For VINT devices, this is the hub serial number.
     *
     * The default is any device.
     */
    // Phidget_setDeviceSerialNumber(ch, <YOUR DEVICE SERIAL NUMBER>);
    
    /*
     * For VINT devices, this specifies the port the VINT device must be plugged into.
     *
     * The default is any port.
     */
    // Phidget_setHubPort(ch, 0);
    
    /*
     * Specifies that the channel should only match a VINT hub port.
     * The only valid channel id is 0.
     *
     * The default is 0 (false), meaning VINT hub ports will never match
     */
    // Phidget_setIsHubPortDevice(ch, 1);
    
    /*
     * Specifies which channel to attach to.  It is important that the channel of
     * the device is the same class as the channel that is being opened.
     *
     * The default is any channel.
     */
    // Phidget_setChannel(ch, 0);
    
    /*
     * In order to attach to a network Phidget, the program must connect to a Phidget22 Network Server.
     * In a normal environment this can be done automatically by enabling server discovery, which
     * will cause the client to discovery and connect to available servers.
     *
     * To force the channel to only match a network Phidget, set remote to 1.
     */
    // PhidgetNet_enableServerDiscovery(PHIDGETSERVER_DEVICEREMOTE);
    // Phidget_setIsRemote(ch, 1);
    
    return EPHIDGET_OK;
}

static void gotAttach(PhidgetHandle phid, void *context) {
    [(__bridge id)context performSelectorOnMainThread:@selector(onAttachHandler)
                                           withObject:nil
                                        waitUntilDone:NO];
}
static void gotDetach(PhidgetHandle phid, void *context) {
    [(__bridge id)context performSelectorOnMainThread:@selector(onDetachHandler)
                                           withObject:nil
                                        waitUntilDone:NO];
}
static void gotError(PhidgetHandle phid, void *context, Phidget_ErrorEventCode errcode, const char *error) {
    
    
    [(__bridge id)context performSelectorOnMainThread:@selector(errorHandler:)
                                           withObject:[NSArray arrayWithObjects:[NSNumber numberWithInt:errcode], [NSString stringWithUTF8String:error], nil]
                                        waitUntilDone:NO];
}
void gotTagData(PhidgetRFIDHandle phid, void *context, const char* tagString, PhidgetRFID_Protocol protocol) {
    NSString *nsTagString = [NSString stringWithUTF8String:tagString];
    @autoreleasepool {
        [(__bridge id)context performSelectorOnMainThread:@selector(onTagHandler:)
                                               withObject:[NSArray arrayWithObjects:[NSString stringWithString:nsTagString], [NSNumber numberWithUnsignedChar:(unsigned char)protocol], nil]
                                            waitUntilDone:NO];
    }
}
void gotTagLostData(PhidgetRFIDHandle phid, void *context, const char* tagString, PhidgetRFID_Protocol protocol) {
    @autoreleasepool {
        [(__bridge id)context performSelectorOnMainThread:@selector(onTagLostHandler)
                                               withObject:nil
                                            waitUntilDone:NO];
    }
}

#pragma mark attach, detach, error, and data events
- (void)onAttachHandler{
    Phidget_DeviceID deviceID;
    int antennaEnabled;
    [phidgetInfoBoxView fillPhidgetInfo:(PhidgetHandle)ch];
    //Get information about device
    Phidget_getDeviceID((PhidgetHandle)ch, &deviceID);
    PhidgetRFID_getAntennaEnabled(ch, &antennaEnabled);
    //Adjust view depending on device
    [readBox setHidden:NO];
    switch(deviceID)
    {
        case PHIDID_1024:
            [writeBox setHidden:NO];
            break;
        case PHIDID_1023:
            [writeBox setHidden:YES];
            break;
        default:
            break;
    }
    
    [antennaEnabledCheck setIntValue:antennaEnabled];
    
    //Adjusting window height
    NSRect frame = [mainWindow frame];
    frame.size.height = [mainWindow maxSize].height;
    [mainWindow setFrame:frame display:NO];
    [mainWindow center];
}

- (void)onDetachHandler{
    //Reset
    [readProtocolText setStringValue:@""];
    [readTagStringText setStringValue:@""];
    [writeTagStringText setStringValue:@""];
    [readBox setHidden:YES];
    [writeBox setHidden:YES];
    [phidgetInfoBoxView fillPhidgetInfo:nil];
    
    
    //Adjusting window height
    NSRect frame = [mainWindow frame];
    frame.size.height = [mainWindow minSize].height;
    [mainWindow setFrame:frame display:NO];
    [mainWindow center];
}

static int errorCounter = 0;
-(void) outputError:(const char *)errorString{
    errorCounter++;
    NSAttributedString *outputString = [[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"%@\n",[NSString stringWithUTF8String:errorString]]];
    [[errorEventLog textStorage] beginEditing];
    [[errorEventLog textStorage] appendAttributedString:outputString];
    [[errorEventLog textStorage] endEditing];
    
    [errorEventLogCounter setIntValue:errorCounter];
    if(![errorEventLogWindow isVisible])
        [errorEventLogWindow setIsVisible:YES];
}

-(void)errorHandler:(NSArray *)errorEventData{
    const char* errorString = [[errorEventData objectAtIndex:1] UTF8String];
    [self outputError:errorString];
}

-(void)addError:(PhidgetReturnCode)result{
    const char *errorString;
    Phidget_getErrorDescription(result, &errorString);
    [self outputError:errorString];
}

-(void)onTagHandler:(NSArray *)tagData{
    NSString *tagString = [tagData objectAtIndex:0];
    PhidgetRFID_Protocol protocol = (PhidgetRFID_Protocol)[[tagData objectAtIndex:1]unsignedCharValue];
    
    [readTagStringText setStringValue:tagString];
    if(protocol == PROTOCOL_EM4100)
        [readProtocolText setStringValue:@"EM4100"];
    else if(protocol == PROTOCOL_ISO11785_FDX_B)
        [readProtocolText setStringValue:@"ISO11785 FDX-B"];
    else if(protocol == PROTOCOL_PHIDGETS)
        [readProtocolText setStringValue:@"PhidgetTag"];
    
}

-(void)onTagLostHandler{
    [readProtocolText setStringValue:@""];
    [readTagStringText setStringValue:@""];
}


#pragma mark GUI controls
- (IBAction)antennaEnabledChecked:(id)sender{
    PhidgetReturnCode result;
    result = PhidgetRFID_setAntennaEnabled(ch, [antennaEnabledCheck intValue]);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
}

- (IBAction)writeString:(id)sender {
    PhidgetReturnCode result;
    long returnValue = 0;
    //check to ensure user wanted to lock tag
    if([lockCheck intValue]){
        NSAlert *alert = [[NSAlert alloc] init];
        [alert addButtonWithTitle:@"OK"];
        [alert addButtonWithTitle:@"Cancel"];
        [alert setMessageText:@"WARNING!\nThis action will lock the RFID tag and prevent further writes. \n\nAre you sure you want to continue? "];
        [alert setAlertStyle:NSAlertStyleCritical];
        returnValue = [alert runModal];
        if(returnValue != NSAlertFirstButtonReturn){
            [lockCheck setIntValue:0];
            return;
        }
    }
    
    PhidgetRFID_Protocol protocol = PROTOCOL_EM4100;
    if([writeProtocolCombo indexOfSelectedItem] == 0)
        protocol = PROTOCOL_EM4100;
    else if([writeProtocolCombo indexOfSelectedItem] == 1)
        protocol = PROTOCOL_ISO11785_FDX_B;
    else if([writeProtocolCombo indexOfSelectedItem] == 2)
        protocol = PROTOCOL_PHIDGETS;
    
    //write new tag
    result = PhidgetRFID_write(ch,(char*)[[writeTagStringText stringValue]UTF8String], protocol, [lockCheck intValue]);
    if(result != EPHIDGET_OK){
        [self addError: result];
        return;
    }
    //reset lock value
    [lockCheck setIntValue:0];
}
@end









